""" grid languages implementation. See http://www.on-signal.org/projects/GRID1 """


from base import Grid
import OSC
import random


__all__ = ["Grid1"]


class Grid1(Grid):


    def __init__(self, cols, rows):
        Grid.__init__(self, cols, rows)
        
        self._state['cursor']  = [0, 0]
        self._state['drawing'] = True
        

    def _warp(self, x, y):

        while x < 0:
            x += self._state['cols']
            y -= 1

        while x >= self._state['cols']:
            x -= self._state['cols']
            y += 1

        while y < 0:
            y += self._state['rows']
            
        y = y % self._state['rows']
        return x, y

    def up(self, *ign):
        self._state['drawing'] = False

    def down(self, *ign):
        self._state['drawing'] = True

    def _walk(self, dx, dy):
        if dx == 0:
            # walk dy
            delta = dy/abs(dy)
            d = 0
            while d != dy:
                if self._state['drawing']:
                    self.set(self._state['cursor'][0], self._state['cursor'][1], not self.get(*self._state['cursor']))
                self._state['cursor'] = list(self._warp(self._state['cursor'][0], self._state['cursor'][1]+delta))
                d += delta

        else:
            # walk dx
            delta = dx/abs(dx)
            d = 0
            while d != dx:
                if self._state['drawing']:
                    self.set(self._state['cursor'][0], self._state['cursor'][1], not self.get(*self._state['cursor']))
                self._state['cursor'] = list(self._warp(self._state['cursor'][0]+delta, self._state['cursor'][1]))
                d += delta
            
        pass

    def north(self, n = 1, *ign):
        self._walk(0, -n)

    def south(self, n = 1, *ign):
        self._walk(0, n)

    def west(self, n = 1, *ign):
        self._walk(-n, 0)

    def east(self, n = 1, *ign):
        self._walk(n, 0)

    def random(self, n = 1, *ign):
        random.choice([self.north,self.south,self.west,self.east])(n)


    def sync(self):
        """ Sync grid also cursor pos """

        Grid.sync(self)

        self._init_osc()

        msg = OSC.OSCMessage("/cursor")
        msg.append(self._state['cursor'][0])
        msg.append(self._state['cursor'][1])
        self._client.send(msg)


if __name__ == "__main__":
    print Grid1
