#
# PIC tools
#


def print_asm(grid, col="G"):
    """ This prints assembler code for generating the bitmap in the given grid. Works only at PICs on 4mhz. """

    lines=480 # the nr of lines on the monitor
    
    s = str(grid)
    print ";    " + s.replace("\n", "\n;    ")

    iterations = lines / grid.get_rows()
    if len(col)==2:
        color = col[0]
    else:
        color = col

    rows = grid.get_rows()

    def dataline(y):
        for x in range(grid.get_cols()):
            if grid.get(x,y):
                print "    PIXEL_%s   ; %2d" % (color, x+1)
            else:
                print "    BLK_%s     ; %2d" % (color, x+1)
        print "    BCF P_HSYNC"
	print "    NOP"
	print "    BSF P_HSYNC"


    print "    MOVLW %d" % iterations
    print "    MOVWF Loop1"

    for y in range(rows):
        if y > rows/2 and len(col)==2:
            color = col[1]
        print "Loop%02d:" % y
        dataline(y)
        print "    NOP"
        print "    DECFSZ Loop1,F"
        print "    GOTO Loop%02d" % y
    
        print "    NOP"
        dataline(y)
        print "    MOVLW %d" % (iterations-1)
        print "    MOVWF Loop1"
        
        if y < rows-1:
            print "    NOP"
            print "    NOP"




def get_mapping(f):
    """ Get a mapping for packing a font character into bytes. """
    
    byte = 0
    bit  = 0

    mapping = []
    for r in range(f._height):
        row = []
        for c in range(f._width):
            row.append( (byte, 2**bit) )
            bit += 1
            if bit == 8:
                byte += 1
                bit = 0
        mapping.append(row)

    if bit == 0:
        numbytes = byte
    else:
        numbytes = byte + 1
    return numbytes, mapping


def print_eeprom_charmap(f):

    data = f.raw_data()
    symbols = data.keys()

    order = [ord(c) for c in data.keys]
    zip(order, data.keys())
    order.sort()

    symbols.sort()
    
    print "; -------------------------"
    print "; Font file %s, %d x %d" % (f._file, f._width, f._height)
    
    print "; PIC EEPROM Character map"
    print
    print "    ORG 2100H"
    print
    print "; %d symbols, in order: " % len(symbols)
    print "; %s" % (", ".join(["'%s'" % s for s in symbols]))
    print
    print "; Jump offset: %d " % len(symbols)
    
    # A hardcoded mapping, because we do not use the 4 corners.
    mapping = [ [ (0,0),  (0,1),  (0,2),  (0,4),   (0,8),  (0,0)  ],
                [ (0,16), (0,32), (0,64), (0,128), (1,1),  (1,2)  ],
                [ (1,4),  (1,8),  (1,16), (1,32),  (1,64), (1,128)],
                [ (2,1),  (2,2),  (2,4),  (2,8),   (2,16), (2,32) ],
                [ (2,64), (2,128),(3,1),  (3,2),   (3,4),  (3,8)  ],
                [ (3,0),  (3,16), (3,32), (3,64),  (3,128), (3,0) ]
                ]
    
    all = {}

    for sym in symbols:
        bytes = [0,0,0,0]
        for l in range(len(data[sym])):
            line = data[sym][l]
            for c in range(len(line)):
                x = line[c]
                bytes[mapping[l][c][0]] += x * mapping[l][c][1]

        all[sym] = bytes

    adr = 0
    i   = 0
    for sym in symbols:
        print "    DE %s     ;  %s  (%d; adr %d)" % (", ".join(["%3d" % x for x in all[sym]]), sym, i, adr)
        adr += len(all[sym])
        i += 1


def print_eeprom_charmap2(f):

    data = f.raw_data()
    symbols = data.keys()
    symbols.sort()
    
    print "; -------------------------"
    print "; Font file %s, %d x %d" % (f._file, f._width, f._height)
    
    print "; PIC EEPROM Character map"
    print
    print "    ORG 2100H"
    print
    print "; Symbols, in order: "
    print "; %s" % (", ".join(["'%s'" % s for s in symbols]))
    print
    mapping = get_mapping(f)
    
    all = {}

    for sym in symbols:
        bytes = [0,0]
        for l in range(len(data[sym])):
            line = data[sym][l]
            for c in range(len(line)):
                x = line[c]
                bytes[mapping[l][c][0]] += x * mapping[l][c][1]

        all[sym] = bytes

    adr = 0
    i   = 0
    for sym in symbols:
        print "    DE %s     ;  %s  (%d; adr %d)" % (", ".join(["%3d" % x for x in all[sym]]), sym, i, adr)
        adr += len(all[sym])
        i += 1

def print_data_charmap(f):
    """ Program-data character map which works using retlw and
    modification of the PCL program counter."""
    
    data = f.raw_data()
    symbols = data.keys()
    symbols.sort()
    
    print "; -------------------------"
    print "; Font file %s, %d x %d" % (f._file, f._width, f._height)
    
    print "; Program-data character map"
    print
    print "; %d symbols, in order: " % len(symbols)
    print "; %s" % (", ".join(["'%s'" % s for s in symbols]))
    print
    print "; Jump offset: %d " % len(symbols)
    print

    numbytes, mapping = get_mapping(f)
    #print "; byte mapping: %s" % repr(mapping)
    
    all = {}

    for sym in symbols:
        bytes = [0] * numbytes
        for l in range(len(data[sym])):
            line = data[sym][l]
            for c in range(len(line)):
                x = line[c]
                bytes[mapping[l][c][0]] += x * mapping[l][c][1]

        all[sym] = bytes

    for n in range(numbytes):
        print "charline%d:" % n
        #print "    addwf PCL,F      ; add offset pc to generate computed goto"
        i = 0
        for sym in symbols:
            print "    retlw %3d        ;  %s  (#%d, byte %d)" % (all[sym][n], sym, i, n)
            i += 1
        

def print_charlookup(f):
    """ Generate character lookup switch statement."""
    
    data = f.raw_data()
    symbols = data.keys()
    symbols.sort()

    i = 0
    for s in symbols:
        print "case %d: return %d;" % (ord(s[0]), i)
        i +=1

