#!/usr/bin/python

# Text generator for PIC+VGA
# resolution = 25x16, letter size = 4x7

from pictools import print_asm

class FontError(Exception):
    pass

class Font:

    _data   = {}
    _width  = None
    _height = None
    _file   = None

    def __init__(self, file):
        self._parse(file)

    def raw_data(self):
        return self._data

    def letters(self):
        return self._data.keys()

    def _parse(self, file):
        self._file = file
        
        lines = "".join(open(file, 'r').readlines()).split("#-")

        width, height = tuple([int(x) for x in lines[0].split('X')])

        letter = "None"
        
        for L in lines[1:]:
            L = L[:-1].split("\n")
            letter = L[0]
            if len(letter) == 0:
                continue
            if letter[0] == "%":
                # special case; if its a %, specify the char as a number.
                if letter[1] == "%":
                    letter = "%"
                else:
                    letter = chr(int(letter[1:]))
                    
            assert len(letter) == 1, "Protocol error: unknown symbol: %s" % letter
            lines = L[1:]
            assert len(lines) == height, "Protocol error: invalid nr of lines in letter %s" % letter
            self._data[letter] = []
            #print letter, [len(x) for x in lines]
            for d in lines:
                row = [0] * width
                for i in range(len(d)):
                    if d[i] == 'X':
                        row[i] = 1
                self._data[letter].append(row)

        self._width  = width
        self._height = height

    def width(self):
        return self._width

    def height(self):
        return self._height

    def render(self, grid, text, pos=(0,0)):

        for y in range(self._height):
            x = pos[0]
            for letter in text:
                for i in range(self._width):
                    if self._data[letter][y][i] == 1:
                        grid.set(x+i, y+pos[1], True)
                x += self._width+1
        return grid

    def substitute(self, text, replace = ""):
        """ Given a string, uppercases it and substitutes all letters
        in the string which are not part of the font by the given
        character."""
        s = []
        text = text.upper()
        for letter in text:
            if letter not in self.letters():
                s.append(replace)
            else:
                s.append(letter)
        return "".join(s)
        


if __name__ == "__main__":
    import grid

    g = grid.Grid(24,16)
    
    f = Font('abc.txt')
    g = f.render(g, "HELLO", (6,0))
    g = f.render(g, "THERE", (0,9))

    print_asm(g, "GB")



