// -------------------------------------------------------------
// animate.cpp
//


// Animations, scrolling, text display.  All kinds of tick functions.
//
// Arjan Scherpenisse <arjan@scherpenisse.net>, June 2009
//

#include "animate.h" // the definitions

extern char _machine[MACHINE_N];      // The machine state
extern char _machine_dirty;          //  The "dirty" flag


void txt_scroll_tick(CTL_Thread *t)
{
    TXT_Scroll *txt = (TXT_Scroll *)t->state;

    // advance the scroller one step
    txt->p += txt->dir;

    // set the machine text
    machine_set(txt->p + txt->offset, txt->text_length, txt->text);

    if (txt->p < -MACHINE_N)
	txt->p += 2*MACHINE_N;
    else if (txt->p > 2*MACHINE_N)
	txt->p -= 2*MACHINE_N;
}


// add a new scrolling text to the machine
CTL_Thread *ctl_thread_add_text_scroll( CTL_Thread *parent, int wakeup, char *text, int offset, int dir)
{
    TXT_Scroll *txt;
    CTL_Thread *thr;
    int l = strlen(text);

    txt = (TXT_Scroll *)malloc(sizeof(TXT_Scroll));

    txt->text = (char *)malloc(l * sizeof(char)); // no need to free this, will be called very infrequently
    strncpy(txt->text, text, l);
    txt->text_length = l;
	
    txt->offset = offset;
    txt->dir    = dir;
#ifdef CFG_MIRRORED
    txt->dir    = -1 * txt->dir;
#endif

    txt->p      = 0;

    thr = ctl_thread_new(&txt_scroll_tick, (void *)txt, wakeup);
    ctl_thread_add(parent, thr);
    return thr;
}


// these character "animations" can probably be done in a better way.
// but Im lazy: don't like string manipulations.
char *t0 = ">  ";
char *t1 = " > ";
char *t2 = "  >";
char *t3 = "  <";
char *t4 = " < ";
char *t5 = "<  ";

void welcome_scroll_tick(CTL_Thread *t)
{
    Welcome_Scroll *wel = (Welcome_Scroll *)t->state;
    
    char* txt;

    // advance the scroller one step
    wel->i = (wel->i + 1) % 3;
    if (wel->dir > 0)
    {
	switch(wel->i) {
	case 0: txt = t0; break;
	case 1: txt = t1; break;
	case 2: txt = t2; break;
	}
    }
    else 
    {
	switch(wel->i) {
	case 0: txt = t3; break;
	case 1: txt = t4; break;
	case 2: txt = t5; break;
	}
    }

    for (int i=wel->from; i<wel->to; i++)
    {
	_machine[i] = txt[i % 3];
    }

    _machine_dirty = 1;
}


CTL_Thread *ctl_thread_add_welcome_scroll( CTL_Thread *parent, int wakeup, int from, int to, int dir)
{
    Welcome_Scroll *wel;
    CTL_Thread *thr;

    wel = (Welcome_Scroll *)malloc(sizeof(Welcome_Scroll));

    wel->from = from;
    wel->to   = to;
    wel->dir  = dir;
    wel->i    = 0;
    wel->dir    = dir;

#ifdef CFG_MIRRORED
    wel->dir    = -1 * wel->dir;
#endif

    thr = ctl_thread_new(&welcome_scroll_tick, (void *)wel, wakeup);
    ctl_thread_add(parent, thr);
    return thr;
}


/// Text blinker

void txt_blink_tick(CTL_Thread *t)
{
    TXT_Blink *txt = (TXT_Blink *)t->state;

    // advance the scroller one step
    txt->i++;

    if (txt->i % 2)
    {
	// set the machine text a
	machine_set(txt->offset, txt->text_length, txt->text_a);
    }
    else
    {
	// set the machine text b when blink is "off"
	machine_set(txt->offset, txt->text_length, txt->text_b);
    }
}


CTL_Thread *ctl_thread_add_text_blink( CTL_Thread *parent, int wakeup, int offset, char *text_a, char *text_b)
{
    TXT_Blink *txt;
    CTL_Thread *thr;
    int l = strlen(text_a);

    txt = (TXT_Blink *)malloc(sizeof(TXT_Blink));

    txt->text_a = (char *)malloc(l * sizeof(char));
    strncpy(txt->text_a, text_a, l);
    txt->text_b = (char *)malloc(l * sizeof(char));
    strncpy(txt->text_b, text_b, l);

    txt->text_length = l;
	
    txt->i      = 0; // initialize simple counter
    txt->offset = offset;

    thr = ctl_thread_new(&txt_blink_tick, (void *)txt, wakeup);
    ctl_thread_add(parent, thr);
    return thr;
}
