// -------------------------------------------------------------
// _oneliner
//
// Self-documenting platform for massive, character-based
// communication using VGA monitors.
//
// Arjan Scherpenisse, <arjan@scherpenisse.net>
// July 2009
//
// Graduation work Gerrit Rietveld Academie, DOGTIME
// Unstable Media / Interactive Design (IDUM) 
//
// All source code and hardware designs for this work are open and
// released under the MIT license (software) and GNU Free
// Documentation License (hardware), respectively.
//
// Project home page:
// http://www.scherpenisse.net/_oneliner/
//
//
// -------------------------------------------------------------
// controller.pde 

// Start with the standard includes
#include <stdio.h>    // is this really needed?
#include <stdlib.h>   // for malloc()
#include <string.h>   // for strlen()

// how many monitors do we have, how many control boards?
#include "config.h"   // = configuration

#include "ctl.h"   // "threading" emulation / control
#include "machine.h" // the state of the machine (character line)
#include "animate.h" // textual animations
#include "eeprom.h"  // eeprom i^2c memory reading

CTL_Thread *start; // the start of our linked list of "threads"
int tick; // the current tick; int; will wrap around, but that's not really an issue.

EEPROM *mem;
/* foo */

int machine_tick_speed = 10;       // The smallest unit of execution; controls the speed (in milliseconds)
int i2c_delay; // update delay for the I^2C communication

#ifdef CFG_I2C
#include <Wire.h>   // include Wire library for I^2C (arduino)

#define		IODIRA		0x00		// MCP23017 address of I/O direction
#define		IODIRB		0x01		// MCP23017 1=input
#define		GPIOA		0x12		// MCP23017 address of GP Value
#define		GPIOB		0x13		// MCP23017 address of GP Value

#define CODE     0x50
#define COMMENTS 0x51

// Do the actual i2c communication.
void i2c_send(char address, char reg, char data)
{
    Wire.beginTransmission(address);
    Wire.send(reg);  // send register first
    Wire.send(data);
    Wire.endTransmission();
}

int i2c_char2code (char ch)
{
    switch (ch) {
#include "char2letter.h" // generated lookup function for the 5x7 font
    }
}

#endif


// The decide_program() function is called once at startup, to
// indicate that the program has reset. It reads the 3 input pins, and
// then decides what program to run.
void decide_program ()
{
    pinMode(2, INPUT);
    pinMode(3, INPUT);
    pinMode(4, INPUT);
    char program = (digitalRead(2) << 2) + (digitalRead(3) << 1) + digitalRead(4);

    switch(program)
    {
    case 0:
	mem = eeprom_init(CODE);

	// the main program -- "sourcecode shower"
	machine_tick_speed = 5000; // once every 5 seconds
	ctl_thread_add(start, ctl_thread_new(&tick_next_sourcecode_line, 0, 1));
	break;
		
    case 1:
	mem = eeprom_init(CODE);

	// the FRANTIC version of the sourcecode display. every 50ms a new line of goodness!
	machine_tick_speed = 50; // omg...
	ctl_thread_add(start, ctl_thread_new(&tick_next_sourcecode_line, 0, 1));
	break;

    case 2:
	mem = eeprom_init(CODE);

	// version of main prog which does everything on one line.
	machine_tick_speed = 5000; // once every 5 seconds
	ctl_thread_add(start, ctl_thread_new(&tick_next_sourcecode_line2, 0, 1));
	break;

    case 3:
	mem = eeprom_init(CODE);
	machine_tick_speed = 50;
	ctl_thread_add(start, ctl_thread_new(&tick_next_sourcecode_line2, 0, 1));
	break;

    case 4:
    {
	// VIDUM welcome greeter
	int d1 = 23; // pos of the first door
	int d2 = 37; // pos of the second door
	
	//machine_set(d1, 5, "VIDUM");
	//machine_set(d2, 5, "VIDUM");
	
	ctl_thread_add_welcome_scroll(start, 10, 0, d1, 1);
	ctl_thread_add_welcome_scroll(start, 10, d1+5, d2, 1);

	ctl_thread_add_text_blink(start, 30, d1, "VIDUM", "*****");
	ctl_thread_add_text_blink(start, 30, d2, "*****", "VIDUM");
	
	ctl_thread_add_welcome_scroll(start, 10, d2+5, 64, -1);
    }
	break;
		
    case 5:
	mem = eeprom_init(COMMENTS);

	// version of main prog which does everything on one line.
	machine_tick_speed = 7000; // once every 5 seconds
	ctl_thread_add(start, ctl_thread_new(&tick_next_sourcecode_line, 0, 1));
	break;
		
    case 6:
	mem = eeprom_init(COMMENTS);

	// version of main prog which does everything on one line.
	machine_tick_speed = 6000; // once every 5 seconds
	ctl_thread_add(start, ctl_thread_new(&tick_next_sourcecode_line, 0, 1));
	break;

    case 7:
	// ABCDEF/012345 test output
	machine_tick_speed = 1000; // once every second
	ctl_thread_add(start, ctl_thread_new(&tick_testmode, 0, 1));
	break;
    }

    // Send a "blink" signal so I'll physically know it has booted.
    // this can probably be done more efficiently ;-)
    digitalWrite(13, 1); delay(50);
    digitalWrite(13, 0); delay(50);
    digitalWrite(13, 1); delay(50);
    digitalWrite(13, 0); delay(50);
    digitalWrite(13, 1); delay(50);
    digitalWrite(13, 0); delay(50);
    digitalWrite(13, 1); delay(50);
    digitalWrite(13, 0); delay(50);
}

void setup ()
{
	
    start = ctl_thread_new(0, 0, 0); // empty starting point

    machine_clear();  // reset the machine to a clean slate.

    decide_program(); // decide what program to run.
	
    tick = 0;

#ifdef CFG_I2C
    Wire.begin();   // start Wire library as I2C-Bus Master

    char board;
    for (board = 0; board < MACHINE_NUM_BOARDS; board++)
    {
	char addr = (0x4 << 3 | board);
	i2c_send(addr, IODIRA, B00000000); // A all output
	i2c_send(addr, IODIRB, B00000000); // B all output
    }
#endif

#ifdef CFG_SERIAL
    Serial.begin(115200); // serial = used for debugging on my laptop
#endif

    digitalWrite(13, 0); // LED off while running! 
    // why do we need a blinking LED?
}

// test-mode tick function
void tick_testmode (CTL_Thread *thr)
{
    static int x = 0;
    if (x)
    {
	machine_set(0, 64, "0123456701234567012345670123456701234567012345670123456701234567");
    }
    else
    {
	machine_set(0, 64, "ABCDEFGHABCDEFGHABCDEFGHABCDEFGHABCDEFGHABCDEFGHABCDEFGHABCDEFGH");
    }
    x = (x+1) % 2;
}

// tick function for displaying the next source code line
void tick_next_sourcecode_line (CTL_Thread *thr)
{
    char *line = eeprom_read_line(mem);
    
    machine_clear();
    machine_set_nowrap(0, strlen(line), line);
    //Serial.println(line); 
    // print the line also on the serial port, for debugging purposes.
    // (commented out because this is now done in machine_sync())

    free(line);					// make sure we cleanup after us! 
    // otherwise the memory will fill up very quicly.
}

int sourcepos = 0;
// alternate version of the sourcecode display: 
// everything on "one line"; e.g. no newlines.
void tick_next_sourcecode_line2 (CTL_Thread *thr)
{
    char *line = eeprom_read_line(mem);
    
    sourcepos = machine_set(sourcepos, strlen(line), line);
    sourcepos = machine_set(sourcepos, 6, "     ");

    // these spaces will be overwritten
    machine_set(sourcepos, 1, "\x81");
    
    free(line); // do not forget to clear memory
    // for installation "stability"
}


// main control loop
void loop ()
{
    CTL_Thread *thr = start;

    // do the linked-list-loop :-)
    do
    {
	if (thr->thread_function && thr->wakeup && !(tick%thr->wakeup)) 
	{
	    thr->thread_function(thr);
	}
	thr = thr->next;
    } while (thr);
    tick++;

    // TODO: isn't it nice to blink?
    //digitalWrite(13, tick%2); // blink the led

    if (_machine_dirty) 
    {
#ifdef CFG_SERIAL
    // debug: synchronize to computer
	Serial.write(0xff); // indicator that data is coming
	Serial.write(MACHINE_N);
	Serial.print(_machine);
#endif

#ifdef CFG_I2C
	// I^2C state synchronization here
	char board = 0; // The current address of the board
	char monitor = 0; // The current monitor
	char i = 0;

	for (monitor=0; monitor<MACHINE_MONITORS_PER_BOARD; monitor++)
	{
	    for (board=0; board<MACHINE_NUM_BOARDS; board++)
	    {
		char addr = (0x4 << 3 | board);

		i = monitor + (MACHINE_MONITORS_PER_BOARD * board);
#ifdef CFG_MIRRORED
		i = MACHINE_NUM_BOARDS - i - 1;
#endif
			
		i2c_send(addr, GPIOA, 0xff);
		i2c_send(addr, GPIOB, 0xff ^ (i2c_char2code(_machine[i])));
		i2c_send(addr, GPIOA, 0xff ^ (1 << monitor));
			
		delayMicroseconds(I2C_DELAY);
	    }
	}
#endif
	_machine_dirty = 0;
    }	
    delay(machine_tick_speed);
}


