// -------------------------------------------------------------
// machine.cpp
//

#include "machine.h"

char _machine_dirty = 0;

char _machine[MACHINE_N];

//
// set the machine to a certain string
//
int machine_set(int x, int l, const char *data)
{
	int i; // just a counter
	int p; // the wrapping point

	int ret; // return value

	//printf("%d\n", x);

	while (x<0)  x += MACHINE_N;  // We need to make sure that X is wrapped
	while (x>=MACHINE_N) x -= MACHINE_N;
	
	
	if (x+l <= MACHINE_N)
	{
		// no wrapping
		for (i=0; i<l; i++)
		{
			_machine[x+i] = data[i];
		}
		ret = x + i;
	}
	else
	{
		// calculate wrapping point
		p = l-((x+l)-MACHINE_N);
		
		for (i=0; i<p; i++)
		{
			_machine[x+i] = data[i];
		}
		ret = x + i;

		for (i=p; i<l; i++)
		{
		    if (i-p >= MACHINE_N) break;
		    _machine[i-p] = data[i];
		    ret = i-p;
		}
	}
	_machine_dirty = 1;

	return ret % MACHINE_N; // fix for out-of-bounds...
}

// set the machine to a certain string, but without wrapping the string.
void machine_set_nowrap(int x, int l, const char *data)
{
	int i; // just a counter
	int p; // the wrapping point

	//printf("%d\n", x);

	while (x<0)  x += MACHINE_N;  // We need to make sure that X is wrapped
	while (x>=MACHINE_N) x -= MACHINE_N;
	
	for (i=0; i<l && i+x < MACHINE_N; i++)
	{
	    _machine[x+i] = data[i];
	}
	_machine_dirty = 1;
}


void machine_sync ()
{
#ifdef SIMULATOR
	printf("%d - [%s]\n", MACHINE_N, _machine);
#else
#ifdef SERIAL
	Serial.println(_machine);
#endif

	// FIXME: I^2C state synchronization here
	
#endif
}




void machine_clear ()
{
	int i;
	for (i=0; i<MACHINE_N; i++) _machine[i] = ' ';
	_machine_dirty = 1;
}
