#include <Wire.h>

void i2c_eeprom_write_byte( int deviceaddress, unsigned int eeaddress, byte data )
{
    int rdata = data;
    Wire.beginTransmission(deviceaddress);
    Wire.send((int)(eeaddress >> 8)); // MSB
    Wire.send((int)(eeaddress & 0xFF)); // LSB
    Wire.send(rdata);
    Wire.endTransmission();
	delay(4); // give the chip some time to recover!
}

// WARNING: address is a page address, 6-bit end will wrap around
// also, data can be maximum of about 30 bytes, because the Wire library has a buffer of 32 bytes
void i2c_eeprom_write_page( int deviceaddress, unsigned int eeaddresspage, byte* data, byte length ) 
{
    Wire.beginTransmission(deviceaddress);
    Wire.send((int)(eeaddresspage >> 8)); // MSB
    Wire.send((int)(eeaddresspage & 0xFF)); // LSB
    byte c;
    for ( c = 0; c < length; c++)
		Wire.send(data[c]);
    Wire.endTransmission();
	delay(4); // give the chip some time to recover!
}

byte i2c_eeprom_read_byte( int deviceaddress, unsigned int eeaddress ) 
{
    byte rdata = 0xFF;
    Wire.beginTransmission(deviceaddress);
    Wire.send((int)(eeaddress >> 8)); // MSB
    Wire.send((int)(eeaddress & 0xFF)); // LSB
    Wire.endTransmission();
    Wire.requestFrom(deviceaddress,1);
    if (Wire.available()) rdata = Wire.receive();
    return rdata;
}

// maybe let's not read more than 30 or 32 bytes at a time!
void i2c_eeprom_read_buffer( int deviceaddress, unsigned int eeaddress, byte *buffer, int length ) 
{
    Wire.beginTransmission(deviceaddress);
    Wire.send((int)(eeaddress >> 8)); // MSB
    Wire.send((int)(eeaddress & 0xFF)); // LSB
    Wire.endTransmission();
    Wire.requestFrom(deviceaddress,length);
    int c = 0;
    for ( c = 0; c < length; c++ )
	{
		if (Wire.available()) buffer[c] = Wire.receive();
	}
}

#define CODE 0x50
#define COMMENTS 0x51

int eeprom_address = CODE; // in binary: 01010000
//int eeprom_address2 = 0x51; // in binary: 01010000

unsigned int current_addr = 0; // the address of the current position
#define MAX_ADDR 65535
#define PAGE_SIZE 24

void setup ()
{
    Wire.begin();

    Serial.begin(115200);

    Serial.println("Send bytes to be written, maximum 65535, 0 to stop.");

    pinMode(0, INPUT);
    pinMode(1, INPUT);
    pinMode(2, INPUT);
}

void debug_read_all ()
{
	byte buf[PAGE_SIZE];
	byte buf_p = 0;

	Serial.println("--dump--");
	unsigned int addr = 0;
	for (addr = 0; addr < MAX_ADDR; addr += PAGE_SIZE)
	{
		int i=0;
		
		i2c_eeprom_read_buffer(eeprom_address, addr, buf, PAGE_SIZE);
		for (i=0; i<PAGE_SIZE; i++)
		{
			if (!buf[i]) {
				// end-marker
				break;
			}
			Serial.print(buf[i]);

		}

		if (!buf[i]) break;
	}
	
	Serial.println("--dump done--");
	Serial.println("");
}

byte writing = 0;

byte buf[PAGE_SIZE];
byte buf_p = 0;

void loop ()
{
	while (Serial.available())
	{
		byte ch = Serial.read();

		if (!writing)
		{
			if (ch == 255)
			{
				Serial.println("--write sequence");
				Serial.println("--");
				current_addr = 0;
				buf_p = 0;
				writing = 1;
				continue;
			}
			
			if (ch == 254)
			{
				Serial.println("Dumping memory");
				// read all memory
				debug_read_all();
				continue;
			}

			Serial.println("not in write mode");
			return;
		}

		//Serial.print(ch);
		i2c_eeprom_write_byte(eeprom_address, current_addr, ch);

		if (ch==0)
		{
			Serial.println("--write done--");
			writing = 0;
		}

		current_addr ++;
		if (!(current_addr % 10))
		{
			Serial.print('.');
		}

		continue;
		
		buf[buf_p] = ch;
		buf_p++;

		if (ch == 0 || current_addr - PAGE_SIZE > MAX_ADDR)
		{
			Serial.println("Finalizing the last bytes.");

			i2c_eeprom_write_page(eeprom_address, current_addr, buf, buf_p);

			Serial.println("Written all bytes.");
			writing = 0;
		}
		else
		{

			if (buf_p == PAGE_SIZE)
			{
				Serial.println("Writing page...");
				i2c_eeprom_write_page(eeprom_address, current_addr, buf, buf_p);
				buf_p = 0;
				current_addr += PAGE_SIZE;
			}

		}
		
	}
}
