;;; ************************************************************
;;; VGA control code for a PIC16F62x running at a 20mhz crystal.
;;; Arjan Scherpenisse <arjan@scherpenisse.net>, may 2009
;;; This code is released in the public domain.
;;; ************************************************************

        LIST P=16F627, R=DEC    ; Use the PIC16F628 and decimal system

        #include "p16f627.inc"  ; Include header file
        
        __config  _EXTCLK_OSC & _LVP_OFF & _WDT_OFF & _PWRTE_ON & _BODEN_ON 

        CBLOCK 0x20             ; Declare variable addresses starting at 0x20
          hloop, vloop		; hsync and vsync loop variables
	
	  dataline 		; data-variable for the b/w pixel data

	  data0, data1, data2, data3, data4, data5, data6
				; 7 data variables for the pixel rows
	
	  charaddr, offset
				; address of current char
        ENDC

#define P_HSYNC PORTA,1
#define P_VSYNC PORTA,0
#define P_GREEN PORTA,3
#define P_BLUE  PORTA,2
	
;
; -----------
; INITIALIZE
; -----------
;
        ORG    0x000           ; Program starts at 0x000

	
        CLRF   PORTA           ; Initialize port A
        CLRF   PORTB           ; Initialize port B

        BSF    STATUS,RP0      ; RAM bank 1

        CLRF   TRISA           ; All pins port A output
        MOVLW   0xFF            
        MOVWF   TRISB           ; PORTB input

        BCF    STATUS,RP0      ; RAM bank 0
;
; ------------------------
; FUNCTION OF PORT A PINS
; ------------------------
;
        MOVLW    7
        MOVWF    CMCON         ; Comparators off, all pins digital I/O

	goto start


	
start:
	
;;; set some data...

	;; checkerboard
;; 	MOVLW b'10101010'
;; 	MOVWF data0
;; 	MOVLW b'01010101'
;; 	MOVWF data1
;; 	MOVLW b'10101010'
;; 	MOVWF data2
;; 	MOVLW b'01010101'
;; 	MOVWF data3
;; 	MOVLW b'10101010'
;; 	MOVWF data4
;; 	MOVLW b'01010101'
;; 	MOVWF data5

	;; sample, for now:
 	movlw 28		; put char in address
 	movwf charaddr	

	
	
;;; VERTICAL SYNC
;;; -------------
;;; visble area = 640 lines
;;; front porch = 16 lines
;;; sync pulse  = 96 lines
;;; back porch  = 48 lines

mainloop:	

        ;; visible area = 640 lines ;; we make 7 pixel rows, 65 per
	;; piece. we need 12 blanks at the top, 13 at the bottom.
	
;;; pixel row 0; 6 pixels = 80 lines
        ;; (data for row 0 is set in the vsync pulse, to save space.
        MOVLW   64
        MOVWF   vloop
	
_v_px0: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px0
	NOP
        
        CALL memhsync

;;; pixel row 1; 6 pixels = 80 lines
	MOVF data1, W
	MOVWF dataline
         
	NOP
	NOP
	
        MOVLW   64
        MOVWF   vloop
	
_v_px1: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px1
	NOP
        
        CALL memhsync

;;; pixel row 2; 6 pixels = 80 lines
	MOVF data2, W
	MOVWF dataline
         
	NOP
	NOP
	
        MOVLW   64
        MOVWF   vloop
	
_v_px2: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px2
	NOP
        
        CALL memhsync

;;; pixel row 3; 6 pixels = 80 lines
	MOVF data3, W
	MOVWF dataline
         
	NOP
	NOP
	
        MOVLW   64
        MOVWF   vloop
	
_v_px3: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px3
	NOP
        
        CALL memhsync

;;; pixel row 4; 6 pixels = 80 lines
	MOVF data4, W
	MOVWF dataline
         
	NOP
	NOP
	
        MOVLW   64
        MOVWF   vloop
	
_v_px4: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px4
	NOP
        
        CALL memhsync
	
;;; pixel row 5; 6 pixels = 80 lines
	MOVF data5, W
	MOVWF dataline
         
	NOP
	NOP
	
        MOVLW   64
        MOVWF   vloop
	
_v_px5: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px5
	NOP
        
        CALL memhsync

;;; pixel row 6; 6 pixels = 80 lines
	MOVF data6, W
	MOVWF dataline
         
	NOP
	NOP
	
        MOVLW   64
        MOVWF   vloop
	
_v_px6: CALL memhsync
	NOP
	NOP
        NOP	
        DECFSZ  vloop,F
        GOTO _v_px6
	NOP
        
        CALL memhsync

	NOP
	NOP
	NOP
	NOP
	
;;; done with the pixels
	
        ;; front porch = 10 lines
        MOVLW   9
        MOVWF   vloop
_v_in3: CALL blankhsync152
        NOP
        DECFSZ  vloop,F
        GOTO _v_in3
	NOP

        CALL blankhsync152
        NOP
        BCF P_VSYNC             ;set vsync port low
        
        ;; vsync pulse = 2 lines
        NOP
        NOP
	CALL blankhsync152

	MOVF data0, W
	MOVWF dataline		;we got some space left, set the data for row 0
	
        BSF P_VSYNC             ;set hsync port high again
        NOP

        ;; back porch = 33 lines
        MOVLW   32
        MOVWF   vloop
_v_in5: CALL blankhsync152
        NOP
        DECFSZ  vloop,F
        GOTO _v_in5
	NOP

	;; load the character data
        CALL loaddatahsync

        GOTO mainloop		; back to the top




;;; HORIZONTAL SYNC
;;; ---------------
;;; One horizontal line is 31.7us -- 158 instructions.
;;; Within this horizontal line:
;;; vis area    - 25.42us = 127 instr
;;; front porch -  0.63us = 3 instr
;;; sync pulse  -  3.81us = 19 ins
;;; back porch  -  1.9us  = 9 ins

;;; for the function, I take 6 instructions out from the backporch,
;;; so there are 152 instructions left. (backporch = 3 ins)
;;; CALL takes up 2 instructions, so there are 6 left to do looping and stuff.


blankhsync152:
        ;; visible area + front porch = 130 instr
        MOVLW   25              ;25*5 = 125
        MOVWF   hloop           ; + these: 125+2 = 127
_h_inner1
	NOP
        NOP
        DECFSZ  hloop,F
        GOTO    _h_inner1          ; Inner loop = 5 usec.
        ;; 3 left

	NOP
	NOP
	NOP
	NOP
        
        ;; now for the hsync pulse
        BCF     P_HSYNC
        ;; 17 inbetween

        MOVLW   3               ;3*5 = 15
        MOVWF   hloop           ; + these two: 15+2 = 17
_h_inner2 NOP
        NOP
        DECFSZ  hloop,F
        GOTO    _h_inner2          ; Inner loop = 5 usec.

        BSF     P_HSYNC

        ;; back porch, nstr
        NOP
        NOP
        NOP
        RETURN
	


loaddatahsync:
        ;; visible area + front porch = 130 instr;
	;; loading data takes 91, so 39 left.

        MOVLW   7              	;7*5=35
        MOVWF   hloop           ; + these: 37
_ld_inner1
	NOP
        NOP
        DECFSZ  hloop,F
        GOTO    _ld_inner1          ; Inner loop = 5 usec.
	NOP
        ;; 2 left
        NOP
	NOP
	
	;; these take up 14*7 = 98 instr
	call t0			;load data for line 0
	movwf data0		;put in databyte 0
	call t1			;etc
	movwf data1
	call t2
	movwf data2
	call t3
	movwf data3
	call t4
	movwf data4
	call t5
	movwf data5
	call t6
	movwf data6
	
        
        ;; now for the hsync pulse
        BCF     P_HSYNC
        ;; 17 inbetween

        MOVLW   3               ;3*5 = 15
        MOVWF   hloop           ; + these two: 15+2 = 17
_ld_inner2 NOP
        NOP
        DECFSZ  hloop,F
        GOTO    _ld_inner2          ; Inner loop = 5 usec.

        BSF     P_HSYNC

        ;; back porch, nstr
        NOP
        NOP
        NOP
        RETURN


	

datahsync152:
        ;; visible area + front porch = 130 instr
        MOVLW   25              ;25*5 = 125
        MOVWF   hloop           ; + these: 125+2 = 127
_hd_inner1
	BCF PORTA,3
	BSF PORTA,3
        DECFSZ  hloop,F
        GOTO    _hd_inner1          ; Inner loop = 5 usec.
        ;; 3 left
        NOP

	NOP
        NOP
        NOP
        
        ;; now for the hsync pulse
        BCF     P_HSYNC
        ;; 17 inbetween

        MOVLW   3               ;3*5 = 15
        MOVWF   hloop           ; + these two: 15+2 = 17
_hd_inner2 NOP
        NOP
        DECFSZ  hloop,F
        GOTO    _hd_inner2          ; Inner loop = 5 usec.

        BSF     P_HSYNC

        ;; back porch, nstr
        NOP
        NOP
        NOP
        RETURN

memhsync:
        ;; visible area + front porch = 130 instr
	;; let's make it 128; so each pixel will take 16 instructions (8 wide)
	;; data has to fit in here....
	NOP

#include px.inc		

        BCF	P_GREEN		; always clear the data at the end of the line
        ;; now for the hsync pulse
        BCF     P_HSYNC
	
        ;; 17 inbetween

	BTFSS	PORTB,7
	GOTO	_filladdress

	NOP
	NOP
	NOP
	NOP
	GOTO 	_cont

_filladdress:
	;; Fill the current address from the data from ports 0-4; only if data is not zero (e.g. all high)
	MOVF	PORTB,W
	XORLW	0xFF		; Invert
	ANDLW	0x7F		; Only look at lower 7 bits
	MOVWF	charaddr
	NOP
_cont:
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

        BSF     P_HSYNC

        ;; back porch, nstr
        NOP
        RETURN	



	;;
t0:	
	movlw HIGH charline0
	movwf PCLATH
	movlw LOW charline0
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL
t1:	
	movlw HIGH charline1
	movwf PCLATH
	movlw LOW charline1
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL
t2:	
	movlw HIGH charline2
	movwf PCLATH
	movlw LOW charline2
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL
t3:	
	movlw HIGH charline3
	movwf PCLATH
	movlw LOW charline3
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL
t4:	
	movlw HIGH charline4
	movwf PCLATH
	movlw LOW charline4
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL
t5:	
	movlw HIGH charline5
	movwf PCLATH
	movlw LOW charline5
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL
t6:	
	movlw HIGH charline6
	movwf PCLATH
	movlw LOW charline6
	addwf charaddr,W
	btfsc STATUS,C
	incf PCLATH,F
	MOVWF PCL

	
;;; The in-program-rom character map

	org 0x200
#include <charset.inc>
	
	end 			; end of program
